import os
import unittest

from frigg_settings.helpers import FileSystemWrapper


class FileSystemWrapperTests(unittest.TestCase):
    def setUp(self):
        self.wrapper = FileSystemWrapper()

    def path(self, *args):
        return os.path.join(os.path.dirname(os.path.dirname(__file__)), *args)

    def test_list_files(self):
        files = self.wrapper.list_files(self.path())

        # This check cannot check the exact files because of
        # generated coverage files.
        self.assertIn('.frigg.yml', files)
        self.assertIn('.gitignore', files)
        self.assertNotIn('frigg_settings', files)
        self.assertNotIn('tests', files)

    def test_read_file(self):
        self.assertEqual(
            self.wrapper.read_file(self.path('MANIFEST.in')),
            'include setup.py README.md MANIFEST.in LICENSE\n'
        )

    def test_file_exist(self):
        self.assertTrue(self.wrapper.file_exist(self.path('setup.py')))
        self.assertFalse(self.wrapper.file_exist(self.path('non-exsting')))
        self.assertFalse(self.wrapper.file_exist(self.path('tests')))



import os
import unittest
from unittest import mock

from frigg_settings import build_settings, build_tasks, load_settings_file
from frigg_settings.helpers import FileSystemWrapper

SETTINGS_FILE = ('tasks:\n - tox -e py34\n - tox -e flake8\n - tox -e isort\n - coverage report -m '
                 '&& coverage xml\n\ncoverage:\n  path: coverage.xml\n  parser: python\n')
SETTINGS_DICT = {'tasks': ['tox'], 'coverage': None}


class SettingsTests(unittest.TestCase):
    def setUp(self):
        self.runner = FileSystemWrapper()

    @mock.patch('frigg_settings.helpers.FileSystemWrapper.list_files')
    def test_build_tasks(self, mock_list_dir):
        build_tasks('path', self.runner)
        mock_list_dir.assert_called_once_with('path')

    @mock.patch('frigg_settings.helpers.FileSystemWrapper.read_file', side_effect=SETTINGS_FILE)
    def test_load_settings_file(self, mock_read_file):
        load_settings_file('.frigg.yml', self.runner)
        mock_read_file.assert_called_once_with('.frigg.yml')

    @mock.patch('frigg_settings.settings.load_settings_file', return_value=SETTINGS_DICT)
    def test_build_settings(self, mock_load_settings_file):
        build_settings(os.path.dirname(os.path.dirname(__file__)), self.runner)
        mock_load_settings_file.assert_called_once()
        self.assertTrue(mock_load_settings_file.call_args_list[0]
                                               .endswith('frigg/frigg-settings/.frigg.yml'))



from os.path import join

import yaml
from frigg_test_discovery import detect_test_tasks


def build_tasks(directory, runner):
    return detect_test_tasks(runner.list_files(directory))


def load_settings_file(path, runner):
    return yaml.load(runner.read_file(path))


def get_path_of_settings_file(directory, runner):
    if runner.file_exist(join(directory, '.frigg.yml')):
        return join(directory, '.frigg.yml')
    elif runner.file_exist(join(directory, '.frigg.yaml')):
        return join(directory, '.frigg.yaml')


def build_settings(directory, runner):
    path = get_path_of_settings_file(directory, runner)

    settings = {
        'webhooks': [],
        'services': []
    }

    if path is not None:
        settings.update(load_settings_file(path, runner))
    else:
        settings['tasks'] = build_tasks(directory, runner)

    if len(settings['tasks']) == 0:
        raise RuntimeError('No tasks found')

    return settings



import os


class FileSystemWrapper(object):
    """
    This is a wrapper around the os library in the stdlib.
    It was created to fit the interface of docker-wrapper-py.
    """

    def list_files(self, path):
        try:
            return [f for f in os.listdir(path) if os.path.isfile(os.path.join(path, f))]
        except OSError:
            return []

    def read_file(self, path):
        with open(path) as f:
            return f.read()

    def file_exist(self, path):
        return os.path.isfile(path)
